/* SPDX-License-Identifier: GPL-2.0
 *
 * Copyright (c) 2020 MediaTek Inc.
 */

#ifndef _DT_BINDINGS_POWER_MT8167_POWER_H
#define _DT_BINDINGS_POWER_MT8167_POWER_H

#define MT8167_POWER_DOMAIN_MM		0
#define MT8167_POWER_DOMAIN_VDEC	1
#define MT8167_POWER_DOMAIN_ISP		2
#define MT8167_POWER_DOMAIN_CONN	3
#define MT8167_POWER_DOMAIN_MFG_ASYNC	4
#define MT8167_POWER_DOMAIN_MFG_2D	5
#define MT8167_POWER_DOMAIN_MFG		6

#endif /* _DT_BINDINGS_POWER_MT8167_POWER_H */
