/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_APQ8016_LPASS_H
#define __DT_APQ8016_LPASS_H

#include <dt-bindings/sound/qcom,lpass.h>

/* NOTE: Use qcom,lpass.h to define any AIF ID's for LPASS */

#endif /* __DT_APQ8016_LPASS_H */
